
package org.ow2.petals.engine.helloworld.listener;

import javax.jbi.messaging.MessagingException;

import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.component.framework.util.UtilFactory;
import org.ow2.petals.engine.helloworld.HelloWorldEngine;
import org.ow2.petals.engine.helloworld.Helloworld;

/**
 * JBI listener for helloworld component
 *
 * @author Olivier FABRE (ofabre) - eBM WebSourcing
 *
 */
public class JBIListener extends AbstractJBIListener {

    private static final String PRINT_MESS = "printMessage";

    private static final String SAY_HELLO = "sayHello";

    private Helloworld helloworld;

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.component.common.listener.AbstractListener#init()
     */
    @Override
    public void init() {
        this.helloworld = ((HelloWorldEngine) this.getComponent()).getHelloworld();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.component.common.listener.AbstractJBIListener#onJBIMessage(org.objectweb.petals.component.common.util.MessageExchangeWrapper)
     */
    @Override
    public boolean onJBIMessage(final Exchange exchange) {
        String outContent = null;
        boolean outResponse = false;
        try {
            // Get Operation name
            final String opName = exchange.getOperationName();

            // Get in Message content
            final String stringContent = UtilFactory.getSourceUtil().createString(
                    exchange.getInMessage().getContent());

            if (PRINT_MESS.equalsIgnoreCase(opName)) {
                if (exchange.isInOnlyPattern()
                        || exchange.isRobustInOnlyPattern()) {
                    this.helloworld.printMessage(stringContent);
                } else {
                    throw new MessagingException("Uncompatible MEP '"
                            + exchange.getPattern().toString()
                            + "' and operation '" + opName + "'");
                }
            } else if (SAY_HELLO.equalsIgnoreCase(opName)) {
                if (exchange.isInOutPattern()
                        || exchange.isInOptionalOutPattern()) {
                    outContent = this.helloworld.sayHello(stringContent);
                } else {
                    throw new MessagingException("Uncompatible MEP '"
                            + exchange.getPattern().toString()
                            + "' and operation '" + opName + "'");
                }
                outResponse = true;
            } else {
                throw new MessagingException("Unknown operation '" + opName
                        + "'");
            }

            if (outResponse) {
                exchange.getOutMessage().setContent(
                        UtilFactory.getSourceUtil().createSource(outContent));
            }
        } catch (final MessagingException e) {
            exchange.setError(e);
        } catch (final PEtALSCDKException e) {
            exchange.setError(e);
        }
        return true;
    }
}
